( cd usr/lib ; rm -rf liborigin.so )
( cd usr/lib ; ln -sf liborigin.so.0.0.4 liborigin.so )
( cd usr/lib ; rm -rf liborigin.so.0.0 )
( cd usr/lib ; ln -sf liborigin.so.0.0.4 liborigin.so.0.0 )
( cd usr/lib ; rm -rf liborigin.so.0 )
( cd usr/lib ; ln -sf liborigin.so.0.0.4 liborigin.so.0 )
